extern crate clap;
extern crate colored;

mod backtrack;

use std::time::SystemTime;

use clap::{load_yaml, App};
use colored::*;

use backtrack::*;

fn main() {
    let yaml = load_yaml!("cli.yml");
    let matches = App::from_yaml(yaml).get_matches();

    let subcommand = matches.subcommand();

    match subcommand.0 {
        "backtrack" => backtrack_solution(subcommand.1.unwrap()),
        _ => panic!("unknown subcommand"),
    }
}

fn backtrack_solution(matches: &clap::ArgMatches<'_>) {
    let verbose = matches.is_present("verbose");
    let first = matches.is_present("first");

    let dim = matches.value_of("dimension");

    let series = matches.values_of("series").map(|v| {
        let values: Vec<_> = v.collect();
        values
    });

    if series.is_some() {
        let ser = series.unwrap();
        let min: usize = ser[0].parse().unwrap();
        let max: usize = ser[1].parse().unwrap();

        for n in min..=max {
            let start = SystemTime::now();

            let found = nreines_first_solution(n, verbose);

            let end = SystemTime::now();

            let duration = end.duration_since(start).unwrap();

            println!(
                "{:0>2} {:>9} in {:<8}ms",
                n.to_string().bold(),
                if found {
                    "found".blue()
                } else {
                    "not found".red()
                },
                (duration.as_secs() * 1000 + u64::from(duration.subsec_millis()))
                    .to_string()
                    .green(),
            );
        }
    } else if dim.is_some() {
        let n: usize = dim.unwrap().parse().unwrap();

        let start = SystemTime::now();

        let mut results: (Option<bool>, Option<usize>) = (Option::None, Option::None);

        if first {
            results.0 = Option::Some(nreines_first_solution(n, verbose));
        } else {
            results.1 = Option::Some(nreines(n, verbose));
        }

        let end = SystemTime::now();

        let duration = end.duration_since(start).unwrap();

        if results.0.is_some() {
            println!(
                "Solution {} in {:>8}ms",
                if results.0.unwrap() {
                    "found".blue()
                } else {
                    "not found".red()
                },
                (duration.as_secs() * 1000 + u64::from(duration.subsec_millis()))
                    .to_string()
                    .green(),
            );
        } else if results.1.is_some() {
            println!(
                "{} solutions found in {:>8}ms",
                results.1.unwrap().to_string().bold(),
                (duration.as_secs() * 1000 + u64::from(duration.subsec_millis()))
                    .to_string()
                    .green(),
            );
        }
    }
}
